﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Media;
using System.IO;
using Unclassified;

namespace RingtoneSimulator
{
	public partial class Form1 : Form
	{
		/// <summary>
		/// The ring tone player. :-)
		/// </summary>
		private SoundPlayer ringtone = new SoundPlayer();
		/// <summary>
		/// DateTime of the next scheduled call, in UTC.
		/// </summary>
		private DateTime nextCall;
		/// <summary>
		/// Duration of the current call.
		/// </summary>
		private TimeSpan callDuration;
		/// <summary>
		/// The source of all randomness.
		/// </summary>
		private Random random = new Random();

		public Form1()
		{
			InitializeComponent();

			Font = SystemFonts.MessageBoxFont;
			Icon = Icon.ExtractAssociatedIcon(Application.ExecutablePath);

			LoadSettings();
		}

		public new DialogResult ShowDialog()
		{
			// Resize the window to its autosized content before it's shown.
			DoLayout();
			return base.ShowDialog();
		}

		private void Form1_Shown(object sender, EventArgs e)
		{
			DoLayout();

			// Find all available ring tones
			try
			{
				foreach (string fileName in Directory.GetFiles(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "Ringtones")))
				{
					TonesCombo.Items.Add(Path.GetFileName(fileName));
				}
			}
			catch (DirectoryNotFoundException)
			{
				MessageBox.Show("The “Ringtones” directory does not exist. No ring tones are available.", "No ring tones found", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			// Select the first ring tone
			if (TonesCombo.Items.Count > 0)
			{
				TonesCombo.SelectedIndex = 0;
			}
		}

		protected override void OnSystemColorsChanged(EventArgs e)
		{
			Font = SystemFonts.MessageBoxFont;
			DoLayout();
		}

		private void MainTable_SizeChanged(object sender, EventArgs e)
		{
			DoLayout();
		}

		private void DoLayout()
		{
			Width = (Width - ClientSize.Width) + tableLayoutPanel1.Width + 2 * tableLayoutPanel1.Left;
			Height = (Height - ClientSize.Height) + tableLayoutPanel1.Height + 2 * tableLayoutPanel1.Top;
		}

		private void ActiveCheckbox_CheckedChanged(object sender, EventArgs e)
		{
			if (ActiveCheckbox.Checked)
			{
				nextCall = GetNextCallTime();
				NextCallLabel.Text = nextCall.ToLocalTime().ToLongTimeString();
			}
			else
			{
				nextCall = DateTime.MaxValue;
				NextCallLabel.Text = "none";
				if (!TestNowCheckbox.Checked)
				{
					ringtone.Stop();
				}
			}
		}

		private void timer1_Tick(object sender, EventArgs e)
		{
			if (ActiveCheckbox.Checked && InTimeRange(DateTime.Now.TimeOfDay))
			{
				// Is it time to hang up?
				if (callDuration != TimeSpan.Zero && DateTime.UtcNow >= nextCall + callDuration)
				{
					ringtone.Stop();
					callDuration = TimeSpan.Zero;

					// Make up the next call time
					nextCall = GetNextCallTime();
					NextCallLabel.Text = nextCall.ToLocalTime().ToLongTimeString();

					TestNowCheckbox.Enabled = true;
				}
				
				// Is it time for the next call?
				if (callDuration == TimeSpan.Zero && DateTime.UtcNow >= nextCall)
				{
					TestNowCheckbox.Checked = false;
					TestNowCheckbox.Enabled = false;

					// Make up how long it's ringing
					callDuration = TimeSpan.FromSeconds((int) MinDurationNum.Value + random.NextDouble() * ((int) MaxDurationNum.Value - (int) MinDurationNum.Value));

					ringtone.PlayLooping();
					NextCallLabel.Text = "NOW until " + (nextCall.ToLocalTime() + callDuration).ToLongTimeString();

					// Log this event
					string logEntry = nextCall.ToLocalTime().ToShortDateString() + " " + nextCall.ToLocalTime().ToLongTimeString() +
						" for " + (int) callDuration.TotalSeconds + " seconds";
					CallLogList.Items.Insert(0, logEntry);
					CallLogList.SelectedIndex = 0;

					try
					{
						File.AppendAllText("call.log", logEntry + Environment.NewLine);
					}
					catch (Exception ex)
					{
						MessageBox.Show("The call log file could not be written.\n\n" + ex.Message, "File write error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
				}
			}
			else
			{
				// Stop it now
				if (!TestNowCheckbox.Checked)
				{
					ringtone.Stop();
				}
			}
		}

		private void TonesCombo_SelectedIndexChanged(object sender, EventArgs e)
		{
			string path = Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "Ringtones");
			ringtone.SoundLocation = Path.Combine(path, TonesCombo.SelectedItem.ToString());
		}

		private void TestNowCheckbox_CheckedChanged(object sender, EventArgs e)
		{
			if (TestNowCheckbox.Checked)
			{
				ringtone.PlayLooping();
			}
			else
			{
				ringtone.Stop();
			}
		}

		private DateTime GetNextCallTime()
		{
			return GetNextCallTime(DateTime.UtcNow);
		}

		private DateTime GetNextCallTime(DateTime baseTime)
		{
			do
			{
				baseTime = baseTime.AddMinutes((int) MinPauseNum.Value + random.NextDouble() * ((int) MaxPauseNum.Value - (int) MinPauseNum.Value));
			}
			while (!InTimeRange(baseTime.ToLocalTime().TimeOfDay));
			return baseTime;
		}

		private bool InTimeRange(TimeSpan timeOfDay)
		{
			TimeSpan start = StartTimePicker.Value.TimeOfDay;
			TimeSpan end = EndTimePicker.Value.TimeOfDay;

			if (start < end)
			{
				// Starts and ends within one day
				return timeOfDay >= start && timeOfDay < end;
			}
			else
			{
				// Runs over night
				return timeOfDay >= start || timeOfDay < end;
			}
		}

		private void LoadSettings()
		{
			string path = Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "config.xml");
			EasyXml xml = new EasyXml();
			if (File.Exists(path))
			{
				xml.Load(path);
			}

			StartTimePicker.Value = DateTime.Today + xml.ReadTimeSpan("/config/time/@start", TimeSpan.FromHours(8));
			EndTimePicker.Value = DateTime.Today + xml.ReadTimeSpan("/config/time/@end", TimeSpan.FromHours(22));
			MinPauseNum.Value = xml.ReadInt("/config/pause/@min", 10);
			MaxPauseNum.Value = xml.ReadInt("/config/pause/@max", 60);
			MinDurationNum.Value = xml.ReadInt("/config/duration/@min", 15);
			MaxDurationNum.Value = xml.ReadInt("/config/duration/@max", 120);
			ActiveCheckbox.Checked = xml.ReadBool("/config/active", false);
			TonesCombo.SelectedItem = xml.ReadString("/config/tone", "");
		}

		private void SaveSettings()
		{
			EasyXml xml = new EasyXml();
			xml.Write("/config/time/@start", StartTimePicker.Value.TimeOfDay);
			xml.Write("/config/time/@end", EndTimePicker.Value.TimeOfDay);
			xml.Write("/config/pause/@min", (int) MinPauseNum.Value);
			xml.Write("/config/pause/@max", (int) MaxPauseNum.Value);
			xml.Write("/config/duration/@min", (int) MinDurationNum.Value);
			xml.Write("/config/duration/@max", (int) MaxDurationNum.Value);
			xml.Write("/config/active", ActiveCheckbox.Checked);
			xml.Write("/config/tone", (TonesCombo.SelectedItem ?? "").ToString());

			string path = Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "config.xml");
			try
			{
				xml.Save(path);
			}
			catch (Exception ex)
			{
				MessageBox.Show("The settings file could not be written.\n\n" + ex.Message, "File write error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void Form1_FormClosed(object sender, FormClosedEventArgs e)
		{
			SaveSettings();
		}

		private void MinPauseNum_ValueChanged(object sender, EventArgs e)
		{
			if (MinPauseNum.Value > MaxPauseNum.Value)
			{
				MaxPauseNum.Value = MinPauseNum.Value;
			}
		}

		private void MaxPauseNum_ValueChanged(object sender, EventArgs e)
		{
			if (MaxPauseNum.Value < MinPauseNum.Value)
			{
				MinPauseNum.Value = MaxPauseNum.Value;
			}
		}

		private void MinDurationNum_ValueChanged(object sender, EventArgs e)
		{
			if (MinDurationNum.Value > MaxDurationNum.Value)
			{
				MaxDurationNum.Value = MinDurationNum.Value;
			}
		}

		private void MaxDurationNum_ValueChanged(object sender, EventArgs e)
		{
			if (MaxDurationNum.Value < MinDurationNum.Value)
			{
				MinDurationNum.Value = MaxDurationNum.Value;
			}
		}
	}
}
